"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2020 LabWare
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
"{ Package: 'jv:libgdbs' }"

"{ NameSpace: Smalltalk }"

GDBLocalProcess subclass:#GDBSimpleProcess
	instanceVariableNames:'debuggerError errorPumpProcess'
	classVariableNames:''
	poolDictionaries:'GDBDebugFlags'
	category:'GDB-Internal'
!

!GDBSimpleProcess class methodsFor:'documentation'!

copyright
"
jv:libgdbs - GNU Debugger Interface Library
Copyright (C) 2015-now Jan Vrany
Copyright (C) 2020 LabWare
Copyright (C) 2022-2023 LabWare

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the 'Software'), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED 'AS IS', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"
!

documentation
"
    GDBStXSimpleProcess is Smalltalk/X specific GDB process implementation
    that uses (two) pipes for communicating with GDB itself. 

    This implementation should work on all platforms, but does not support
    access to GDB's built-in CLI - this means that clients wanting to provide
    a CLI interface to the GDB must emulate it on their own.

    [author:]
        Jan Vrany <jan.vrany@fit.cvut.cz>

    [instance variables:]

    [class variables:]

    [see also:]
        GDBRemoteProcess
        GDBStXUnixProcess

"
! !

!GDBSimpleProcess methodsFor:'error pump'!

errorPumpLoop
    [ debuggerError atEnd ] whileFalse:[
        debuggerError readWait.
        debuggerError atEnd ifFalse:[
            | line |

            line := debuggerError nextLine.
            line notNil ifTrue:[ 
                Logger log: line severity: #error facility: 'GDB'
            ].
        ].
    ]

    "Created: / 15-01-2018 / 09:31:39 / jv"
!

errorPumpStart
    errorPumpProcess isNil ifTrue:[
        errorPumpProcess := [
                TraceEvents ifTrue:[
                    Logger log: 'error pump: starting' severity: #trace facility: 'GDB'
                ].
                [ 
                    self errorPumpLoop
                ] ensure: [ 
                    TraceEvents ifTrue:[
                        Logger log: 'error pump: terminated' severity: #trace facility: 'GDB'
                    ].
                    errorPumpProcess := nil. 
                ].
            ] newProcess.
        errorPumpProcess name:('GDB Error pump (%1)' bindWith: self id).
        errorPumpProcess priority:Processor userBackgroundPriority.
        errorPumpProcess resume.
    ].

    "Created: / 15-01-2018 / 09:28:06 / jv"
    "Modified: / 20-10-2018 / 06:53:19 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 14-07-2023 / 09:59:55 / Jan Vrany <jan.vrany@labware.com>"
!

errortPumpStop
    | t |

    t := errorPumpProcess.
    (t notNil and:[ t isDead not]) ifTrue:[ 
        errorPumpProcess := nil.
        t terminate.
         "/ raise its prio to make it terminate quickly
        t priority:(Processor userSchedulingPriority + 1)                       
    ].

    "Created: / 15-01-2018 / 09:29:49 / jv"
! !

!GDBSimpleProcess methodsFor:'initialization & release'!

initializeWithCommand: aString
    | inputPipe  input  outputPipe  output errorPipe error argv |

    inputPipe := GDBPortlib current newPipe.
    input := inputPipe second.
    outputPipe := GDBPortlib current newPipe.
    output := outputPipe first.
    errorPipe := GDBPortlib current newPipe.
    error := outputPipe first.   

    command := aString.
    argv := (self command2argv: command) ,
            ((Array new: 5)
             at: 1 put: '-q';
             at: 2 put: '--interpreter';
             at: 3 put: 'mi';
             at: 4 put: '-iex';
             at: 5 put: 'set new-console on';
             yourself).

    pid := GDBPortlib current 
                 spawn: argv
                 stdin: inputPipe first
                 stdout: outputPipe second
                 stderr: errorPipe second
                 exit: [:stat | self exited: stat ].
    pid isNil ifTrue:[
        input close.
        output close.
        error close.
        self error:'Failed to launch gdb'.
    ].

    inputPipe first close.
    outputPipe second close.
    errorPipe second close.

    debuggerInput := input.
    debuggerOutput := output.
    debuggerError := error.

    "Created: / 12-12-2018 / 20:13:01 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 11-06-2019 / 11:45:00 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified (comment): / 25-06-2019 / 21:10:02 / jv"
    "Modified: / 25-07-2023 / 12:52:46 / Jan Vrany <jan.vrany@labware.com>"
!

release
    pid := connection := nil.
    debuggerInput notNil ifTrue:[ debuggerInput close ].
    debuggerOutput notNil ifTrue:[ debuggerOutput close ].

    "Created: / 20-06-2014 / 21:35:25 / Jan Vrany <jan.vrany@fit.cvut.cz>"
    "Modified: / 15-12-2017 / 23:59:35 / Jan Vrany <jan.vrany@fit.cvut.cz>"
! !

!GDBSimpleProcess class methodsFor:'documentation'!

version_HG

    ^ '$Changeset: <not expanded> $'
! !

